///////////////////////////////////////////////////////////////////////////////
//! \file fcBaseCAN.h:
//! \brief Header File for the fcBase CAN API.
//!
//! Use this header file for developing user defined FlexCard CAN applications.
//! See also printed documentation for more information. The data types and
//! enumerations can be found in the header file "fcBaseTypesCAN.h".
//!
//! Generally:
//!
//! - Each function of this library (except some error handling functions)
//!   returns an error code. If the return value is equal to 0, no error
//!   occured. A number greater than 0 indicates an error. To get more
//!   information about it, use the error handling functions.
//!
//! See also printed documentation for more information.
//!
//! \par Overview:
//! Author:         WO/SH/Ba/TB/Tr/LJ
//! $Date$ <br>
//! \par Copyright:
//! Copyright 2009-2015 STAR COOPERATION GmbH. All rights reserved.
//! \version $Revision$
///////////////////////////////////////////////////////////////////////////////

#ifndef _FCBASE_CAN_H_
#define _FCBASE_CAN_H_

#ifndef _FCBASE_TYPES_H_
	#error Please include first the header files "fcBaseTypes.h"
#endif 

#ifndef _FCBASE_TYPES_CAN_H_
	#error Please include first the header files "fcBaseTypesCAN.h"
#endif

// Linux
#ifdef __linux__

	#ifndef FCBASE_API
		#define FCBASE_API
	#endif
	//! Please do not use the FCBASE_EXPORTS define in your application, unless
	//! you have the source code ;-)
	#ifdef FCBASE_EXPORTS
		#define EXPORTDLL extern "C" __attribute__ ((visibility ("default")))
	#else
		#if defined(__cplusplus)
			#define EXPORTDLL extern "C"
		#else
			#define EXPORTDLL 
		#endif
	#endif

// Windows
#else

	#ifndef FCBASE_API
		#define FCBASE_API __stdcall
	#endif
	//! Please do not use the FCBASE_EXPORTS define in your application, unless
	//! you have the source code ;-)
	#ifdef FCBASE_EXPORTS
		#define EXPORTDLL extern "C" __declspec(dllexport)
	#else
		#if defined(__cplusplus)
			#define EXPORTDLL extern "C" __declspec(dllimport)
		#else
			#define EXPORTDLL __declspec(dllimport)
		#endif
	#endif

#endif



//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//
//          CAN
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------




//-----------------------------------------------------------------------------
//           Initialization 
//-----------------------------------------------------------------------------

//! This function is used to start the monitoring of a CAN bus. Once called,
//! the function changes the communication controller state from configuration
//! state to normal active state. The current communication controller state
//! can be read using the function #fcbGetCcState. If the FlexCard is started
//! the function #fcbGetCcState will return the value #fcStateNormalActive.
//! 
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> Index of the CAN communication controller.
//! \param resetTimestamps [IN] <br> Set this parameter to 0 to restart
//!		   the measurement without resetting the FlexCard timestamp. Set it to
//!		   <> 0 to start the measurement from the beginning. The timestamps
//!		   have micro second resolution.
//! \param mode [IN] <br> The monitoring mode. See #fcCANMonitoringMode for
//! 	   details which monitoring mode is supported.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         please use the functions described in the section Error Handling to
//!         get extended error information.
//! \remarks After the monitoring has started, the user should check if the
//!          integration in the cluster was successful, #fcbCANGetCcState should
//!          return the state #fcCANCcStateNormalActive. 
//! \note After the monitoring has successfully started, the receive process has
//! 	  to be started as soon as possible to avoid an overflow (error packet
//! 	  fcErrFlexcardOverflow is received). Once an overflow occurred, no more
//! 	  packets can be received. The monitoring has to be stopped and started
//! 	  again.
//! \note On a FlexCard PXIe3/PCIe3 in the RBS Monitoring mode, the driver checks
//!       how the FlexConfig RBS running on the processor is configured. The
//!       CAN decoder supports the frame format fcCANFDFormatIso11898_1, but not
//!       the frame format fcCANFDFormatBoschSpecV1_0. If the FlexConfig RBS
//!       is configured to a not supported CAN FD frame format, this function
//!       returns FUNCTION_NOT_IMPLEMENTED.
//! \see #fcbCANMonitoringStop, #fcbCANGetCcState, #fcCANMonitoringMode
EXPORTDLL fcError FCBASE_API fcbCANMonitoringStart(fcHandle hFlexCard,
												   fcCC CC,
												   fcBool resetTimestamps,
												   fcCANMonitoringMode mode);


//! This function stops the CAN bus measurement of the selected communication
//! controller. The communication controller is set back in its configuration state.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> Index of the CAN communication controller.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to get
//!         extended error information.
//! \see #fcbCANMonitoringStart
EXPORTDLL fcError FCBASE_API fcbCANMonitoringStop(fcHandle hFlexCard,
												  fcCC CC);


//! This function returns the current CAN communication controller state. For a
//! description of possible states, refer to the enumeration #fcCANCcState.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> Index of the CAN communication controller.
//! \param pState [OUT] <br> Current CAN communication controller state.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         please use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcbMonitoringStart, #fcbMonitoringStop
EXPORTDLL fcError FCBASE_API fcbCANGetCcState(fcHandle hFlexCard,
											  fcCC CC,
											  fcCANCcState* pState);


//-----------------------------------------------------------------------------
//          Configuration
//-----------------------------------------------------------------------------

//! This function configures a CAN communication controller. This function
//! cannot be called during monitoring. Before the configuration of the CAN
//! CC starts, all CAN message buffers and the CAN transmit FIFO are reset.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> CAN communication controller identifier.
//! \param cfg [IN] <br> CAN communication controller configuration parameters.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcCANCcConfig
EXPORTDLL fcError FCBASE_API fcbCANSetCcConfiguration(fcHandle hFlexCard,
													  fcCC CC,
													  fcCANCcConfig cfg);


//! This function configures the message buffers of the CAN communication controller.
//! Configuring message buffers is allowed in all communication controller states.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> CAN communication controller identifier.
//! \param bufferNumber [IN] <br> Identifier of the message buffer to be configured. Valid values are 1 to 128.
//! \param cfg [IN] <br> Message buffer configuration parameters.
//! \param ignoreTxRqstLock [IN] <br> Set to <> 0 to configure the message buffer also
//!        if the previous transmittable buffer was not transmitted yet.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcbCANGetMessageBuffer, #fcCANBufCfg
EXPORTDLL fcError FCBASE_API fcbCANSetMessageBuffer(fcHandle hFlexCard,
													fcCC CC,
													fcDword bufferNumber,
													fcCANBufCfg cfg,
													fcBool ignoreTxRqstLock);


//! This function reads a specific CAN message buffer configuration.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> CAN communication controller identifier.
//! \param bufferNumber [IN] <br> Identifier of the message buffer to be read. Valid values are 1 to 128.
//! \param pCfg [OUT] <br> The configuration parameters of the specified message buffer.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcbCANSetMessageBuffer, #fcCANBufCfg
EXPORTDLL fcError FCBASE_API fcbCANGetMessageBuffer(fcHandle hFlexCard,
													fcCC CC,
													fcDword bufferNumber,
													fcCANBufCfg* pCfg);

//! This function configures the transmit FIFO feature a for CAN communication
//! controller. With the transmit FIFO it's possible to transmit CAN data
//! messages without the configuration of several message buffer objects. 
//! This function cannot be called during monitoring. The FlexCard driver 
//! corrects the transmit interval if the configured CAN baudrate is too low. 
//! After calling fcbCANSetTxFifoConfiguration call fcbCANGetTxFifoConfiguration 
//! to get the corrected transmit interval.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> CAN communication controller identifier.
//! \param cfg [IN] <br> CAN transmit FIFO configuration parameters.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcCANTxFifoConfig, #fcbCANGetTxFifoConfiguration, #fcbCANTxFifoTransmit
EXPORTDLL fcError FCBASE_API fcbCANSetTxFifoConfiguration(fcHandle hFlexCard,
														  fcCC CC,
														  fcCANTxFifoConfig cfg);

//! This function reads the transmit FIFO configuration parameters of a CAN
//! communication controller. With the transmit FIFO it's possible to transmit
//! CAN data messages without the configuration of several message buffer objects.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> CAN communication controller identifier.
//! \param pCfg [OUT] <br> CAN transmit FIFO configuration parameters.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcCANTxFifoConfig, #fcbCANSetTxFifoConfiguration, #fcbCANTxFifoTransmit
EXPORTDLL fcError FCBASE_API fcbCANGetTxFifoConfiguration(fcHandle hFlexCard,
														  fcCC CC,
														  fcCANTxFifoConfig* pCfg);

//! This function resets the CAN Communication Controller transmit FIFO buffer. 
//! CAN data messages in the FIFO are lost.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> CAN communication controller identifier.
//! \param bOnlyCurrent [IN] <br> Reset only the current CAN message that is scheduled
//!								  for transmission by the CAN CC. The next CAN message
//!								  is going to be scheduled for transmission. Current
//!								  CAN data message is lost. Other queued CAN data
//!								  messages in the FIFO aren't lost.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcCANTxFifoConfig, #fcbCANTxFifoTransmit
EXPORTDLL fcError FCBASE_API fcbCANTxFifoReset(fcHandle hFlexCard,
											   fcCC CC,
											   fcBool bOnlyCurrent);

//! This function configures the CAN or CANFD frame ids accepted by the device driver.
//! Only the configured ids which are forwarded to the user application, all other 
//! CAN or CANFD frames are rejected. This function can be used on CAN bus or CANFD bus 
//! and depend on type of communication controler. 
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> Index of the CAN/CANFD communication controller.
//! \param id [IN] <br> FrameId, which will be accepted.
//! \param bExt [IN] <br> Flag for extended Id
//!                       If bExt is true, the extended Id will be used. Id has 29 bit length.
//!                       If bExt is false, the extended Id will not be used. Id has 11 bit length.
//! \param bCanFd[IN] <br> Flag for CANFD frame
//!                       If bCanFd is true, this frame is a CANFD Frame. 
//!                       If bCanFd is false, this frame is not a CANFD Frame.
//! \param reset [in] Flag for reset all filter.
//!                       If reset is true , then all configured filter will be reset.
//!                       If reset is false, no filter will be reset. New filter will be added to existing filter table. 
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to get
//!         extended error information.
//! \see #fcCC, #fcChannel, #fcbReceive
EXPORTDLL fcError FCBASE_API fcbCANSetFilterConfiguration(fcHandle hFlexCard, 
                                                           fcCC CC,
                                                           fcDword id, 
                                                           fcBool bExt, 
                                                           fcBool bCanFd, 
                                                           fcBool reset);

//-----------------------------------------------------------------------------
//          Transmit
//-----------------------------------------------------------------------------

//! This function writes the data bytes in a CAN communication controller transmit
//! buffer of the FlexCard. The message should normally be transmitted as soon as
//! possible. In case the transmission of any number of message buffers may be
//! requested at the same time, they are transmitted subsequently according to their
//! priority (The message object numbers are configurable from 1 up to 128, the lower
//! is the message number, the higher is the priority). If the transmit acknowledgment
//! is activated, a CAN packet with a direction flag = 1 (Tx) is generated as soon as
//! the message has been transmitted. This function should only be called when the
//! FlexCard is in normal active state or when all message buffer configurations have
//! been done.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> CAN communication controller identifier.
//! \param bufferNumber [IN] <br> The number of the message buffer used for the transmission.
//! \param data [IN] <br> The data to be transmitted. The configured DLC in the message
//!		   buffer determinates the size of bytes which will be copied in the transmit buffer.
//! \param transmitNewData [IN] <br> Set to <> 0 to update the data of the message
//!		   buffer. Set to 0 if the previous data shall be sent again.
//! \param ignoreTxRqstLock [IN] <br> Set to <> 0 to transmit the message although
//!        if the previous message was not transmitted yet.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcbCANGetMessageBuffer, #fcCANBufCfg
EXPORTDLL fcError FCBASE_API fcbCANTransmit(fcHandle hFlexCard,
											fcCC CC,
											fcDword bufferNumber,
											fcByte data[8],
											fcBool transmitNewData,
											fcBool ignoreTxRqstLock);

//! This function writes a CAN data message into the CAN transmit FIFO buffer of 
//! the FlexCard. It holds maximum 512 messages. The function returns immediately 
//! and does not wait for the data frame to arrive on the bus. The message should 
//! normally be transmitted as soon as possible. This function should only be called 
//! when the CAN transmit FIFO is configured. In case the FIFO is full, the error
//! code TX_FIFO_FULL is returned.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> CAN communication controller identifier.
//! \param id [IN] <br> The CAN identifier. Valid values for a standard CAN Id
//!						range from 0x0 to 0x7FF. Valid values for an extended
//!						CAN Id range from 0x0 to 0x1FFFFFFF.
//! \param bExt [IN] <br> Set to <> 0 to transmit a CAN message with an extended
//!						  identifier. Set to 0 to transmit a standard CAN message.
//! \param pData [IN] <br> The data to be transmitted.
//! \param dlc [IN] <br> The size of bytes which will be copied in the FIFO.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcbCANSetTxFifoConfiguration
EXPORTDLL fcError FCBASE_API fcbCANTxFifoTransmit(fcHandle hFlexCard,
												  fcCC CC,
												  fcDword id,
												  fcBool bExt,
												  fcByte* pData,
												  fcDword dlc);






//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//
//          CAN FD
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
//          Configuration
//-----------------------------------------------------------------------------

//! This function configures a CAN FD communication controller. This function
//! cannot be called during monitoring.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> CAN FD communication controller identifier.
//! \param cfg [IN] <br> CAN FD communication controller configuration parameters.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcCANFDCcConfig
EXPORTDLL fcError FCBASE_API fcbCANFDSetCcConfiguration(fcHandle hFlexCard,
													  fcCC CC,
													  fcCANFDCcConfig cfg);


//-----------------------------------------------------------------------------
//          Transmit
//-----------------------------------------------------------------------------

//! This function writes the data bytes in a CAN FD communication controller of the FlexCard.
//! The message should normally be transmitted as soon as possible. The FlexCard
//! internally uses a Tx FIFO which holds maximum 32 messages. The messages appear on the
//! bus in the order in which they were transmitted by the user. If the transmit acknowledgment
//! is activated, a CAN FD packet with a direction flag = 1 (Tx) is generated as soon as
//! the message has been transmitted. This function should only be called when the
//! FlexCard is in normal active state. If the internal Tx message fifo is full,
//! the error TX_FIFO_FULL is returned and the message is not transmitted.
//! Sending CAN Remote frames via the function fcbCANFDTransmit is not supported.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> CAN communication controller identifier.
//! \param pFrame [IN] <br> The frame that should be transmitted. This struct
//!        contains the payload data and configuration options.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcCANFDTxFrame
EXPORTDLL fcError FCBASE_API fcbCANFDTransmit(fcHandle hFlexCard,
											fcCC CC,
                                            const fcCANFDTxFrame* pFrame
                                            );




#endif /* #define _FCBASE_CAN_H_ */
