﻿///////////////////////////////////////////////////////////////////////////////
//! \brief Header File for the fcBase Ethernet API.
//!
//! Use this header file for developing user defined FlexCard Ethernet applications.
//! See also printed documentation for more information. The data types and
//! enumerations can be found in the header file "fcBaseTypesEthernet.h".
//!
//! Generally:
//!
//! - Each function of this library (except some error handling functions)
//!   returns an error code. If the return value is equal to 0, no error
//!   occured. A number greater than 0 indicates an error. To get more
//!   information about it, use the error handling functions.
//!
//! See also printed documentation for more information.
//!
//! \par Overview:
//! Author:         WO/SH/Ba/TB/Tr/LJ
//! $Date: 2018-12-18 18:30:06 +0100 (Di, 18 Dez 2018) $ <br>
//! \par Copyright:
//! Copyright 2009-2020 STAR COOPERATION GmbH. All rights reserved.
//! \version $Revision: 18948 $
///////////////////////////////////////////////////////////////////////////////

#ifndef _FCBASE_ETH_H_
#define _FCBASE_ETH_H_

#ifndef _FCBASE_TYPES_H_
	#error Please include first the header files "fcBaseTypes.h"
#endif 

#ifndef _FCBASE_TYPES_ETH_H_
	#error Please include first the header files "fcBaseTypesEth.h"
#endif

// Linux
#ifdef __linux__

	#ifndef FCBASE_API
		#define FCBASE_API
	#endif
	//! Please do not use the FCBASE_EXPORTS define in your application, unless
	//! you have the source code ;-)
	#ifdef FCBASE_EXPORTS
		#define EXPORTDLL extern "C" __attribute__ ((visibility ("default")))
	#else
		#if defined(__cplusplus)
			#define EXPORTDLL extern "C"
		#else
			#define EXPORTDLL 
		#endif
	#endif

// Windows
#else

	#ifndef FCBASE_API
		#define FCBASE_API __stdcall
	#endif
	//! Please do not use the FCBASE_EXPORTS define in your application, unless
	//! you have the source code ;-)
	#ifdef FCBASE_EXPORTS
		#define EXPORTDLL extern "C" __declspec(dllexport)
	#else
		#if defined(__cplusplus)
			#define EXPORTDLL extern "C" __declspec(dllimport)
		#else
			#define EXPORTDLL __declspec(dllimport)
		#endif
	#endif

#endif


//-----------------------------------------------------------------------------
//           Initialization 
//-----------------------------------------------------------------------------

//! This function is used to start the monitoring of an ethernet channel. 
//! Set the configuration with #fcbEthSetCcConfiguration before calling this function.
//! 
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> Index of the ethernet controller.
//! \param resetTimestamps [IN] <br> Set this parameter to 0 to restart
//!		   the measurement without resetting the FlexCard timestamp. Set it to
//!		   <> 0 to start the measurement from the beginning. The timestamps
//!		   have micro second resolution.
//! \param mode [IN] <br> The monitoring mode. See #fcEthMonitoringMode for
//! 	   details which monitoring mode is supported.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         please use the functions described in the section Error Handling to
//!         get extended error information.
//! \note After the monitoring has successfully started, the receive process has
//! 	  to be started as soon as possible to avoid an overflow (error packet
//! 	  fcErrFlexcardOverflow is received). Once an overflow occurred, no more
//! 	  packets can be received. The monitoring has to be stopped and started
//! 	  again.
//! \see #fcbEthMonitoringStop, #fcEthMonitoringMode
EXPORTDLL fcError FCBASE_API fcbEthMonitoringStart(fcHandle hFlexCard,
												   fcCC CC,
												   fcEthMonitoringMode mode,
												   fcBool resetTimestamps);


//! This function stops the ethernet channel measurement of the selected
//! controller. 
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> Index of the ethernet controller.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to get
//!         extended error information.
//! \see #fcbEthMonitoringStart
EXPORTDLL fcError FCBASE_API fcbEthMonitoringStop(fcHandle hFlexCard,
												  fcCC CC);

//! This function configures the selected ethernet controller.
//! Call this function before calling #fcbEthMonitoringStart.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> Index of the ethernet controller.
//! \param cfg [IN] <br> Ethernet configuration parameters.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to get
//!         extended error information.
//! \see #fcCC
EXPORTDLL fcError FCBASE_API fcbEthSetCcConfiguration(fcHandle hFlexCard, fcCC eCC,  fcEthCcConfig cfg);

//! This function configures the ethernet filter for the selected ethernet controller. Only the ethernet
//! frames which configured by setting filter are forwarded, all other ethernet frames are rejected. See
//! the configuration notice for further details. This function can also be called while the ethernet
//! monitoring is started.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> Index of the ethernet controller.
//! \param combinedFilter [IN] Flag of combined filter.
//!         If value is 1, the filter is a combined filter.
//!         If value is 0, the filter is not a combined filter.
//! \param filterCfg [IN] <br> Filter configuration parameters.
//! \param reset [in] Flag for resetting all filters.
//!         If value is 1, all configured filters will be reset.
//!         If value is 0, no filter will be reset. New filter will be added to the existing filter table.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to get
//!         extended error information.
//! \see #fcCC
EXPORTDLL fcError FCBASE_API fcbEthSetFilterConfiguration(fcHandle hFlexCard, fcCC cc, fcBool combinedFilter, fcEthFilterConfig filterCfg, fcBool reset);

//! This function transmits a raw ethernet frame for the selected ethernet controller.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> Index of the ethernet controller.
//! \param pRawEthernetFrame [IN] <br> The ethernet raw frame to transmit. This includes the following fields (Layer 2 Ethernet frame without Frame check sequence (32-bit CRC):
//!          MAC Destination                               : 6 octets
//!          MAC Source                                    : 6 octets
//!          802.1Q tag (optional)                         : 4 octets
//!          Ethertype (Ethernet II) or length (IEEE 802.3): 2 octets
//!          Payload                                       : 46‑1500 octets
//! \param lengthOfEthernetFrame [IN] <br> The size of the ethernet frame in number of octets.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to get
//!         extended error information.
//! \see #fcCC
EXPORTDLL fcError FCBASE_API fcbEthTransmit(fcHandle hFlexCard,
	fcCC CC,
	const fcByte* pRawEthernetFrame,
	fcWord lengthOfEthernetFrame);



#endif /* #define _FCBASE_ETH_H_ */
