///////////////////////////////////////////////////////////////////////////////
//! \file fcBasePMC.h:
//! \brief Header File for the fcBase PMC API.
//!
//! Use this header file for developing user defined FlexCard PMC applications.
//! See also printed documentation for more information. The data types and
//! enumerations can be found in the header file "fcBaseTypesPMC.h".
//!
//! Generally:
//!
//! - Each function of this library (except some error handling functions)
//!   returns an error code. If the return value is equal to 0, no error
//!   occurred. A number greater than 0 indicates an error. To get more
//!   information about it, use the error handling functions.
//!
//! \par Overview:
//! Author:         WO/SH/Ba/TB/Tr/LJ
//! $Date$ <br>
//! \par Copyright:
//! Copyright 2009-2015 STAR COOPERATION GmbH. All rights reserved.
//! \version $Revision$
///////////////////////////////////////////////////////////////////////////////

#ifndef _FCBASE_PMC_H_
#define _FCBASE_PMC_H_

#ifndef _FCBASE_TYPES_H_
    #error Please include first the header file "fcBaseTypes.h"
#endif 

#ifndef _FCBASE_TYPES_PMC_H_
    #error Please include first the header file "fcBaseTypesPMC.h"
#endif 

// Linux
#ifdef __linux__

	#ifndef FCBASE_API
		#define FCBASE_API
	#endif
	//! Please do not use the FCBASE_EXPORTS define in your application, unless
	//! you have the source code ;-)
	#ifdef FCBASE_EXPORTS
		#define EXPORTDLL extern "C" __attribute__ ((visibility ("default")))
	#else
		#if defined(__cplusplus)
			#define EXPORTDLL extern "C"
		#else
			#define EXPORTDLL 
		#endif
	#endif

// Windows
#else

	#ifndef FCBASE_API
		#define FCBASE_API __stdcall
	#endif
	//! Please do not use the FCBASE_EXPORTS define in your application, unless
	//! you have the source code ;-)
	#ifdef FCBASE_EXPORTS
		#define EXPORTDLL extern "C" __declspec(dllexport)
	#else
		#if defined(__cplusplus)
			#define EXPORTDLL extern "C" __declspec(dllimport)
		#else
			#define EXPORTDLL __declspec(dllimport)
		#endif
	#endif

#endif


//! Gets information about the FlexTiny modules mounted on the FlexCard.
//! Depending on the hardware type this is the FlexTiny II or FlexTiny III type.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param pTinyInfo [OUT] <br> This struct contains the FlexTiny information.
//!         The caller must provide memory for this parameter.
//! \param pNumberOfTinySlots [OUT] <br> The number of FlexTiny slots that
//!        are available on the hardware. The caller must provide memory for this parameter.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         please use the functions described in the section Error Handling to
//!         get extended error information.
//! \note This function is not available for FlexCard Cyclone II (SE) and FlexCard USB-M.
EXPORTDLL fcError FCBASE_API fcbGetTinyInfo(fcHandle hFlexCard,
												fcTinyInfoCollection* pTinyInfo,
                                                fcByte* pNumberOfTinySlots);


//! Sets the bus termination individually for each communication controller channel.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param BusType [IN] <br> The bus type termination.
//! \param CC [IN] <br> Index of the bus type communication controller.
//! \param Channel [IN] <br> FlexRay channel(s) concerned by the termination. This
//!		   parameter needs only be set for fcBusTypeFlexRay.
//! \param bTermination [IN] <br> Set the value <> 0 enables the bus termination.
//!		   Set the value 0 disables the bus termination.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         please use the functions described in the section Error Handling to
//!         get extended error information.
EXPORTDLL fcError FCBASE_API fcbSetBusTerminationCc(fcHandle hFlexCard,
													fcBusType BusType,
													fcCC CC,
													fcChannel Channel,
													fcBool bTermination);

//! Reads the bus termination individually for each communication controller channel.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param BusType [IN] <br> The bus type termination.
//! \param CC [IN] <br> Index of the bus type communication controller.
//! \param Channel [IN] <br> FlexRay channel(s) concerned by the termination. This
//!		   parameter needs only be set for fcBusTypeFlexRay. Valid values are
//!		   fcChannelA and fcChannelB.
//! \param pbTermination [OUT] <br> The current bus termination. The value 0 indicates
//!		   a disabled termination. A value <> 0 indicates an enabled termination.
//!        The caller must provide memory for this parameter.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         please use the functions described in the section Error Handling to
//!         get extended error information.
EXPORTDLL fcError FCBASE_API fcbGetBusTerminationCc(fcHandle hFlexCard,
													fcBusType BusType,
													fcCC CC,
													fcChannel Channel,
													fcBool* pbTermination);

//! Sets the bus termination individually for each hardware bus channel.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param BusChannel [IN] <br> The bus channel to terminate.
//! \param BusType [IN] <br> The bus type termination.
//! \param bTermination [IN] <br> Set the value <> 0 enables the bus termination.
//!		   Set the value 0 disables the bus termination.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         please use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcbGetBusTermination, #fcBusChannel, #fcBusType
EXPORTDLL fcError FCBASE_API fcbSetBusTermination(fcHandle hFlexCard,
												  fcBusChannel BusChannel,
												  fcBusType BusType,
												  fcBool bTermination);

//! Reads the current bus termination individually for each hardware bus channel.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param BusChannel [IN] <br> The bus channel to terminate.
//! \param BusType [IN] <br> The bus type termination.
//! \param pbTermination [OUT] <br> The current bus termination. The value 0 indicates
//!		    a disabled termination. A value <> 0 indicates an enabled termination.
//!         The caller must provide memory for this parameter.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         please use the functions described in the section Error Handling to
//!         get extended error information.
EXPORTDLL fcError FCBASE_API fcbGetBusTermination(fcHandle hFlexCard,
												  fcBusChannel BusChannel,
												  fcBusType BusType,
												  fcBool* pbTermination);

//! THIS FUNCTION IS OBSOLETE
//! This function allows to select one of the 2 available communication controllers
//! of the FlexCard PMC. Following called functions relate to this selected
//! communication controller.
//! \param hFlexCard [IN] <br> Handle to a FlexCard
//! \param CCIndex [IN] <br> The CC index to be set.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         please use the functions described in the section Error Handling to
//!         get extended error information. 
//! \see #fcbGetCcIndex
EXPORTDLL fcError FCBASE_API fcbSetCcIndex(fcHandle hFlexCard, 
                                           fcCC CCIndex);

//! THIS FUNCTION IS OBSOLETE
//! This function reads and returns the index of the selected communication 
//! controller.
//! \param hFlexCard [IN] <br> Handle to a FlexCard
//! \param pCCIndex [OUT] <br> The current selected CC index.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcbSetCcIndex
EXPORTDLL fcError FCBASE_API fcbGetCcIndex(fcHandle hFlexCard, 
                                           fcCC* pCCIndex);

//! This function is used for the FlexCard PMC-II. It reads information about
//! a firmware slot. The PMC II can load 8 different firmwares, called user images.
//! The user images are stored in 8 image slots on the hardware. A user image
//! may be activated by fcbFWSelectImage. The changes take effect after a
//! complete shut down of the PC. Whether an image slot is active at the moment
//! can be checked with the bit bIsActive in the struct pFWInfo.
//! \param hFlexCard [IN] <br> Handle to a FlexCard
//! \param index [IN] <br> The selected image slot. Valid range: 1 to 8.
//! 0 is reserved.
//! \param pFWInfo [OUT] <br> The information about the image slot.
//!         The caller must provide memory for this parameter.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to
//!         get extended error information.
EXPORTDLL fcError FCBASE_API fcbFWGetImageInfo(fcHandle hFlexCard,
											   fcDword index,
											   fcFWInfo* pFWInfo);

//! This function is used for the FlexCard PMC-II. It selects
//! a firmware slot. The PMC II can load 8 different firmwares, called user images.
//! The user images are stored in 8 image slots on the hardware. Information about
//! an image slot may be read with fcbFWGetImageInfo. The changes take effect
//! after a complete shut down of the PC.
//! \param hFlexCard [IN] <br> Handle to a FlexCard
//! \param index [IN] <br> The selected image slot. Valid range: 1 to 8.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to
//!         get extended error information.
EXPORTDLL fcError FCBASE_API fcbFWSelectImage(fcHandle hFlexCard,
											  fcDword index);


#endif /* #define _FCBASE_PMC_H_ */
