/*/////////////////////////////////////////////////////////////////////////////
//! \file fcBaseTypesEth.h:
//! \brief Header file for the data types and enumerations of the fcBase Ethernet API.
//!
//! Use this header file for developing user defined FlexCard applications. This
//! header file contains all structures, data types and enumerations which are
//! used in the fcBase Ethernet API. For a description of the API function, please
//! refer to the header file "fcBaseEth.h".
//!
//! See also printed documentation for more information.
//!
//! \par Overview:
//! $Date: 2018-12-18 18:30:06 +0100 (Di, 18 Dez 2018) $ <br>
//! \par Copyright:
//! Copyright 2019 STAR COOPERATION GmbH. All rights reserved.
//! \version $Revision: 18948 $
/////////////////////////////////////////////////////////////////////////////*/

#ifndef _FCBASE_TYPES_ETH_H_
#define _FCBASE_TYPES_ETH_H_

//-----------------------------------------------------------------------------
//           Initialization 
//-----------------------------------------------------------------------------

//! This enumeration defines the different modes available that are used to monitor an
//! ethernet bus.
//! \see #fcbEthMonitoringStart
typedef enum fcEthMonitoringMode
{
	fcEthMonitoringPassive = 0, //!< Monitors the ethernet bus in passive mode without impacting the real bus traffic.
												
} fcEthMonitoringMode;

//! This enumeration describes Ethernet Mode, that includes the following information:
//! Data Transfer Rate, Connection Cable Type, Transmission Direction (relevant to automotive Ethernet)
//! Some Modes are only usable for Ethernet FlexTinys while others for automotive Ethernet FlexTinys
//! Abbreviations used are:
//! Base        stands for base band transmission 
//! T1/TX       stands for type of connection cable used
//! 10/100/1000 stands for 10/100/1000 Mbps (Mega bits per second) data transfer rate
//! Master      stands for Master-Slave direction (relevant to automotive Ethernet)
//! Slave       stands for Slave-Master direction
//! Auto        stands for Auto Negotiation (relevant to Ethernet). In this Mode Channel
//!             decides the suitable data transfer rate based on negotiation.
//! Fixed       stands for fixed data transfer used (No Auto Negotiation)
typedef enum fcEthMode
{
    fcEthMode100BaseT1Master = 1,	//< 100BASE-T1 Master-Slave Transmission Direction
    fcEthMode100BaseT1Slave,		//< 100BASE-T1 Slave-Master Transmission Direction
    fcEthMode1000BaseT1Master,		//< 1000BASE-T1 Master-Slave Transmission Direction
    fcEthMode1000BaseT1Slave,		//< 1000BASE-T1 Slave-Master Transmission Direction
    fcEthModeAuto,					//< Ethernet Mode: Auto Negotiation, possible rates used 10/100/1000 Mbps
    fcEthMode1000BaseTXFixed,		//< Ethernet Mode: 1000 Mbps 
    fcEthMode100BaseTXFixed,		//< Ethernet Mode: 100 Mbps
    fcEthMode10BaseTXFixed,			//< Ethernet Mode: 10 Mbps
} fcEthMode;

typedef struct fcEthCcConfig
{
    fcEthMode Mode;          //< Ethernet Mode indicates the data transfer rate, the connection cable type
                             //< and in case of automotive Ethernet the master/slave setting. Some Modes relate
                             //< to Ethernet while others to automotive Ethernet
    fcBool PromiscuousMode;	 //< Promiscuous mode (Currently Not Supported)
                             //< True:  Channel receives all frames Without MAC Address Check
                             //< False: Channel receives frames from specific MAC Addresses
    fcByte MAC[6];           //< MAC (Media Access Control) Address of Hardware Channel
    fcByte VLANCount;        //< Number of VLANs (Virtual Local Area Networks)
    fcWord* VLANs;           //< Array of VLAN Ids (Virtual Local Area Networks) must be provided in order to 
                             //< be able to receive and send from these VLANs on the Channel
} fcEthCcConfig;

typedef struct fcEthFilterConfig
{
    struct
    {
        fcByte FlagDstMac : 1; ///< Flag if destination MAC filter should be set
        fcByte FlagSrcMac : 1; ///< Flag if source MAC filter should be set
        fcByte FlagVlanId : 1; ///< Flag if VLAN filter should be set
        fcByte FlagEtherType : 1; ///< Flag if EtherType filter should be set
        fcByte FlagDstIpv4 : 1; ///< Flag if destination IP filter should be set
        fcByte FlagSrcIpv4 : 1; ///< Flag if source IP filter should be set
        fcByte FlagDstPort : 1; ///< Flag if desination port filter should be set
        fcByte FlagSrcPort : 1; ///< Flag if source IP filter should be set
    }Flags;

    fcByte Reserved;        ///< Reserved must be set to 0
    fcByte DstMac[6];       ///< Detination MAC filter entry
    fcByte SrcMac[6];       ///< Source MAC filter entry
    fcWord VlanId;          ///< VLAN filter entry
    fcWord EtherType;       ///< EtherType filter entry
    fcByte DstIpv4[4];      ///< Destination IP filter entry
    fcByte SrcIpv4[4];      ///< Source IP filter entry
    fcWord DstPort;         ///< Destination port filter entry
    fcWord SrcPort;         ///< Source port filter entry
} fcEthFilterConfig;

#endif  /* #define _FCBASE_TYPES_ETH_H_ */
