/*/////////////////////////////////////////////////////////////////////////////
//! \file fcBaseTypesUARTOverCAN.h:
//! \brief Header file for the data types and enumerations of the fcBase UART over CAN API.
//!
//! Use this header file for developing user defined FlexCard applications. This
//! header file contains all structures, data types and enumerations which are
//! used in the fcBase UART Over CAN API. For a description of the API function, please
//! refer to the header file "fcBaseUARTOverCAN.h".
//!
//! See also printed documentation for more information.
//!
/////////////////////////////////////////////////////////////////////////////*/

#ifndef _FCBASE_TYPES_UARTOVERCAN_H_
#define _FCBASE_TYPES_UARTOVERCAN_H_



//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//
//          UART over CAN
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------





//-----------------------------------------------------------------------------
//           Initialization 
//-----------------------------------------------------------------------------

//! This enumeration selects the number of data bits per character that the 
//! peripheral will transmit and receive.
//! \see #fcUARTOverCANCcConfig
typedef enum fcUARTOverCANDataLength
{
	fcUARTOverCANDataLength5Bits = 0,						//!< 5 data bits per character
	fcUARTOverCANDataLength6Bits,							//!< 6 data bits per character
	fcUARTOverCANDataLength7Bits,							//!< 7 data bits per character
	fcUARTOverCANDataLength8Bits,							//!< 8 data bits per character
	fcUARTOverCANDataLength9Bits,							//!< 9 data bits per character
} fcUARTOverCANDataLength;

//! Number of stop bits. This is used to select the number of stop bits per character
//! that the peripheral will transmit and receive. Note that regardless of the number
//! of stop bits selected the receiver will only check the first stop bit.
//! For 5 data bits per character it is 1.5 stop bits, not 2.
//! \see #fcUARTOverCANCcConfig
typedef enum fcUARTOverCANStopBit
{
	fcUARTOverCAN1StopBit,						//!< 1 stop bit
	fcUARTOverCAN2StopBit,						//!< 2 stop bits
} fcUARTOverCANStopBit;

//! Enable and disable parity generation and detection in a transmitted and received data character.
//! Odd: An odd number of logic '1's is transmitted or checked.
//! Even: An even number of logic '1's is transmitted or checked.
//! Mark: The parity bit is always transmitted and checked as set.
//! Space: The parity bit is always transmitted and checked as cleared.
//! No: Disable parity generation and detection in a transmitted and received data character.
typedef enum fcUARTOverCANParityBit
{
	fcUARTOverCANOddParity,
	fcUARTOverCANEvenParity,
	fcUARTOverCANMarkParity,
	fcUARTOverCANSpaceParity,
	fcUARTOverCANNoParity,
} fcUARTOverCANParityBit;

//-----------------------------------------------------------------------------
//          Configuration
//-----------------------------------------------------------------------------

//! This structure describes the configuration of a UART over CAN communication controller.
//! \see #fcbUARTOverCANSetCcConfiguration, #fcbUARTOverCANGetCcConfiguration
typedef struct fcUARTOverCANCcConfig
{
	fcDword BaudRate;					//!< Defines the baud rate. Valid values
									//!< are from 200000 (200kBd) to 2000000 (2MBd).
	fcUARTOverCANDataLength DataLength;
	fcUARTOverCANStopBit StopBits;
	fcUARTOverCANParityBit Parity;
    fcDword PauseInNs;					//!< Delay between transmitted bytes. Resolution is 30ns.
    //!< E.g. when you specify the value 40, the hardware uses 30ns.
    //!< The value that the hardware uses can be read via #fcbUARTOverCANGetCcConfiguration.
    fcDword Reserved[2];					//!< Reserved for future use.
} fcUARTOverCANCcConfig;

//! This structure contains a UART frame that can be transmitted by a UART
//! communication controller.
typedef struct fcUARTOverCANTxFrame
{
	fcByte DLC;						//!< Defines the data length to be transmitted.
	fcByte reserved1[3];					//!< Reserved for alignment
	fcByte data[32];					//!< Defines the data for transmission. All of the 32 data
									//!< bytes can be set. The corresponding DLC parameter is
									//!< used to define the number of bytes to transmit.
									//!< Valid values are from 1 to 32.
	fcDword reserved2;					//!< Reserved for future use.
} fcUARTOverCANTxFrame;


//! This structure contains a UART frame that can be transmitted by a UART
//! communication controller.
typedef struct fcUARTOverCANTxFrameV2
{
    fcDword DLC;						//!< Defines the data length in bytes to transmit.
                                        //!< The start of the data is set in the variable data.
                                        //!< Valid values are from 1 to 1024.
    fcDword reserved;					//!< Reserved for future use.
    fcByte* data;					//!< Pointer to the payload to transmit.
} fcUARTOverCANTxFrameV2;

#endif  /* #define _FCBASE_TYPES_UARTOVERCAN_H_ */
