///////////////////////////////////////////////////////////////////////////////
//! \file fcBaseUARTOverCAN.h:
//! \brief Header File for the fcBase UARTOverCAN API.
//!
//! Use this header file for developing user defined FlexCard UARTOverCAN applications.
//! See also printed documentation for more information. The data types and
//! enumerations can be found in the header file "fcBaseTypesUARTOverCAN.h".
//!
//! Generally:
//!
//! - Each function of this library (except some error handling functions)
//!   returns an error code. If the return value is equal to 0, no error
//!   occured. A number greater than 0 indicates an error. To get more
//!   information about it, use the error handling functions.
//!
//! See also printed documentation for more information.
//!
//! \par Overview:
//! \par Copyright:
//! Copyright 2009-2015 STAR COOPERATION GmbH. All rights reserved.
//! \version $Revision: 18358 $
///////////////////////////////////////////////////////////////////////////////

#ifndef _FCBASE_UARTOVERCAN_H_
#define _FCBASE_UARTOVECAN_H_

#ifndef _FCBASE_TYPES_H_
	#error Please include first the header files "fcBaseTypes.h"
#endif 

#ifndef _FCBASE_UARTOVECAN_H_
	#error Please include first the header files "fcBaseTypesUARTOverCAN.h"
#endif

// Linux
#ifdef __linux__

	#ifndef FCBASE_API
		#define FCBASE_API
	#endif
	//! Please do not use the FCBASE_EXPORTS define in your application, unless
	//! you have the source code ;-)
	#ifdef FCBASE_EXPORTS
		#define EXPORTDLL extern "C" __attribute__ ((visibility ("default")))
	#else
		#if defined(__cplusplus)
			#define EXPORTDLL extern "C"
		#else
			#define EXPORTDLL 
		#endif
	#endif

// Windows
#else

	#ifndef FCBASE_API
		#define FCBASE_API __stdcall
	#endif
	//! Please do not use the FCBASE_EXPORTS define in your application, unless
	//! you have the source code ;-)
	#ifdef FCBASE_EXPORTS
		#define EXPORTDLL extern "C" __declspec(dllexport)
	#else
		#if defined(__cplusplus)
			#define EXPORTDLL extern "C" __declspec(dllimport)
		#else
			#define EXPORTDLL __declspec(dllimport)
		#endif
	#endif

#endif



//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//
//          UARTOverCAN
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------




//-----------------------------------------------------------------------------
//           Initialization 
//-----------------------------------------------------------------------------

//! This function is used to start the monitoring of a UARTOverCAN bus.
//! 
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> Index of the UARTOverCAN communication controller.
//! \param restartTimestamps [IN] <br> Set this parameter to 0 to restart
//!		   the measurement without resetting the FlexCard timestamp. Set it to
//!		   <> 0 to start the measurement from the beginning. The timestamps
//!		   have micro second resolution.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         please use the functions described in the section Error Handling to
//!         get extended error information.
//! \note After the monitoring has successfully started, the receive process has
//! 	  to be started as soon as possible to avoid an overflow (error packet
//! 	  fcErrFlexcardOverflow is received). Once an overflow occurred, no more
//! 	  packets can be received. The monitoring has to be stopped and started
//! 	  again.
//! \see #fcbUARTOverCANMonitoringStop
EXPORTDLL fcError FCBASE_API fcbUARTOverCANMonitoringStart(fcHandle hFlexCard,
												   fcCC CC,
												   fcBool resetTimestamps);


//! This function stops the UARTOverCAN bus measurement of the selected communication
//! controller. The communication controller is set back in its configuration state.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> Index of the UARTOverCAN communication controller.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to get
//!         extended error information.
//! \see #fcbUARTOverCANMonitoringStart
EXPORTDLL fcError FCBASE_API fcbUARTOverCANMonitoringStop(fcHandle hFlexCard,
												  fcCC CC);

//-----------------------------------------------------------------------------
//          Configuration
//-----------------------------------------------------------------------------

//! This function configures an UARTOverCAN communication controller. This function
//! cannot be called during monitoring.
//! The desired baud rate chosen in the fcUARTOverCANCcConfig configuration may not
//! be set exactly because of rounding errors. The actually used baud rate can be
//! read with fcbUARTOverCANGetCcConfiguration.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> UARTOverCAN communication controller identifier.
//! \param cfg [IN] <br> UARTOverCAN communication controller configuration parameters.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcUARTOverCANCcConfig
EXPORTDLL fcError FCBASE_API fcbUARTOverCANSetCcConfiguration(fcHandle hFlexCard,
													  fcCC CC,
													  fcUARTOverCANCcConfig cfg);

//! This function reads the configuration of an UARTOverCAN communication controller.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> UARTOverCAN communication controller identifier.
//! \param cfg [OUT] <br> UARTOverCAN communication controller configuration parameters.
//!         The caller must provide memory for this parameter.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcUARTOverCANCcConfig
EXPORTDLL fcError FCBASE_API fcbUARTOverCANGetCcConfiguration(fcHandle hFlexCard,
													  fcCC CC,
													  fcUARTOverCANCcConfig* cfg);


//-----------------------------------------------------------------------------
//          Transmit
//-----------------------------------------------------------------------------

//! This function writes the data bytes in a UARTOverCAN communication controller of the FlexCard.
//! The message should normally be transmitted as soon as possible.
//! 1 to 32 bytes can be transmitted. Every byte received is in a separate packet to timestamp all bytes.
//! New applications should use the function #fcbUARTOverCANTransmitV2.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> UARTOverCAN communication controller identifier.
//! \param pFrame [IN] <br> The frame that should be transmitted. This struct
//!        contains the payload data and configuration options.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcUARTOverCANTxFrame
EXPORTDLL fcError FCBASE_API fcbUARTOverCANTransmit(fcHandle hFlexCard,
											fcCC CC,
                                            const fcUARTOverCANTxFrame* pFrame
                                            );


//! This function writes the data bytes in a UARTOverCAN communication controller of the FlexCard.
//! The message should normally be transmitted as soon as possible.
//! 1 to 1024 bytes can be transmitted. Every byte received is in a separate packet to timestamp all bytes.
//! \param hFlexCard [IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> UARTOverCAN communication controller identifier.
//! \param pFrame [IN] <br> The frame that should be transmitted. This struct
//!        contains the payload data.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcUARTOverCANTxFrameV2
EXPORTDLL fcError FCBASE_API fcbUARTOverCANTransmitV2(fcHandle hFlexCard,
    fcCC CC,
    const fcUARTOverCANTxFrameV2* pFrame
);




#endif /* #define _FCBASE_UARTOVECAN_H_ */
